
; Definitionen fr Mega88

; Belegungen der Atmel-Ports:

;*****************************************************************


;		PB0		ICP1/CLKO		Eingang externe Triggerauswertung, H Trigger
;   PB1		OC1A				Clock fr Ram-Adresszhler			
;   PB2		OC1B/SS			Enable Trigger
;   PB3		OC2A/MOSI
;   PB4		MISO				Shift-Daten
;   PB5		SCK					Shift-Clock
				
;   PC0		SCL					Daten 0
;   PC1		SDA					Daten 1
;   PC2		TCK					Clock-Mux A
;   PC3		IMS					Clock-Mux B
;		PC4		TDO					Umschaltung Sample/Lesen, H Lesen, L Sample 
;		PC5		TDI		 			Input-Buffer TriState H, aktiv L

;   PD0		RxD					RS232
;   PD1		TxD					RS232
;   PD2		INT0				Daten 2		
;   PD3		INT1/OC2B		Daten 3
;   PD4		XCK/T0			Daten 4		
;   PD5		OC0B				Daten 5
;   PD6		OC0A/AIN0		Daten 6	
;   PD7		AIN1				Daten 7

;   Byte 1	Mode
;   Byte 2	Triggerbyte
;   Byte 3	Triggermaske
;   Byte 4	Timerintervall
;   Byte 5	Pretrigger		1...8, 12,5%, 25%, ... 87,5%, 0% (8)


;Mode0: internal Clock, internal Trigger

;	kompletten Ram nach dem Samplen kommentarlos zum PC schicken 

; 0xF0 wird fter als maximale Steuerbyte-Anzahl vom PC gesendet als Software-Reset
; 

;*****************************************************************

;--- Allgemeines -----------------------------------------------

.equ    F_CPU           = 20000000
.equ    CYCLES_PER_US   = ((F_CPU+500000)/1000000)

.equ  RAM_SIZE	= 0x32								; genutzter Speicher in kB als Packed BCD
.equ  BUF_LEN_MAX = 32768
.equ  RAM_CLOCK = 50                 ; 80 fr 80MHz, 50 fr 50MHz

.equ    UART_BAUD_RATE  = 500000
;.equ    UART_BAUD_SELECT= (F_CPU/(UART_BAUD_RATE*16))-1
.equ    UART_BAUD_SELECT= (F_CPU/(UART_BAUD_RATE*8))-1			; U2X gesetzt

;--- Daten ------------------------------------------------

.equ	DATEN_1_DDR		= DDRC
.equ	DATEN_1_PORT	= PORTC
.equ	DATEN_1_IN		= PINC
.equ	DATEN_1_MASKE	= 0b00000011					; D0 und D1 sind an PC0 und PC1

.equ	DATEN_2_DDR		= DDRD
.equ	DATEN_2_PORT	= PORTD
.equ	DATEN_2_IN		= PIND
.equ	DATEN_2_MASKE	= 0b11111100

.equ	CTRL_PORT			= PORTC

.equ	MUX_A					= PC2
.equ	MUX_B					= PC3
.equ	C_SAMPLE			= PC4									; Umschaltung Sample/Lesen, H Lesen, L Sample 
.equ	C_BUFFER			= PC5									; Input-Buffer TriState H, aktiv L

.equ	CLOCK_PORT		= PORTB
.equ	TRIGGER_PIN		= PINB

.equ	TRIG_IN				= PB0									; Eingang externe Triggerauswertung, H Trigger
.equ	C_CLOCK				= PB1									; Takt fr Ram-Adresszhler (OC2B) 
.equ  TRIG_ENABLE   = PB2

.equ  SHIFT_PORT    = PORTB

.equ	SHIFT_DATA		= PB4
.equ	SHIFT_CLOCK		= PB5

.equ	CMD_RESTART		= 0xF0                ; auf Anfang

.equ	MODE_MAX		= 12									  ; maximale Anzahl Funktionen in der Tabelle

.equ	CLK_STOP		= (1<<WGM12)						; Timer Stop, CTC-Mode
.equ	CLK_START		= (1<<WGM12) | (1<<CS10); Timer Start, Prescale 1

.equ	MUX_MASKE		= 0b11110011

.if RAM_CLOCK == 50

.equ	MUX_INT			= 0b00000000
.equ	MUX_20MHZ		= 0b00000100
.equ	MUX_50MHZ		= 0b00001000
.equ	MUX_EXT			= 0b00001100

.equ	CLK_50M			= 0		; -> 100ns
.equ	CLK_20M			= 0		; -> 100ns
.equ	CLK_10M			= 0		; -> 100ns
.equ	CLK_5M			= 1		; -> 200ns
.equ	CLK_2M			= 4		; -> 500ns
.equ	CLK_1M			= 9		; -> 1s
.equ	CLK_500k		= 19	; -> 2
.equ	CLK_200k		= 49	; -> 5s
.equ	CLK_100k		= 99	; -> 10s
.equ	CLK_50k			= 199	; -> 20s
.equ	CLK_20k			= 499	; -> 50s
.equ	CLK_10k			= 999	; -> 100s
.equ	CLK_5k			= 1999	; -> 200s
.equ	CLK_2k			= 4999	; -> 500s
.equ	CLK_1k			= 9999	; -> 1ms

.else

.equ	MUX_INT			= 0b00000000
.equ	MUX_20MHZ		= 0b00000100
.equ	MUX_40MHZ		= 0b00001000
.equ	MUX_80MHz		= 0b00001100

.equ	CLK_80M			= 0		; -> 100ns
.equ	CLK_40M			= 0		; -> 100ns
.equ	CLK_20M			= 0		; -> 100ns
.equ	CLK_10M			= 0		; -> 100ns
.equ	CLK_5M			= 1		; -> 200ns
.equ	CLK_2M			= 4		; -> 500ns
.equ	CLK_1M			= 9		; -> 1s
.equ	CLK_500k		= 19	; -> 2
.equ	CLK_200k		= 49	; -> 5s
.equ	CLK_100k		= 99	; -> 10s
.equ	CLK_50k			= 199	; -> 20s
.equ	CLK_20k			= 499	; -> 50s
.equ	CLK_10k			= 999	; -> 100s
.equ	CLK_5k			= 1999	; -> 200s
.equ	CLK_2k			= 4999	; -> 500s
.equ	CLK_1k			= 9999	; -> 1ms

.endif

;**************************** Ram-Reservierungen ******************

;--- Internes Ram ------------------------------------------------

.DSEG

BUF_LEN_L:	.byte			1			; Samplebufferlnge
BUF_LEN_H:	.byte 		1

.;**************************** Register-Bits **************************

;--- lower Register -----------------------------------------------

.def  MUL_L         = r0
.def  MUL_H         = r1

.def  BUF_LEN_REG   = r2

.def	NULL					= r3

.def	PRE_LEN_L			= r5
.def	PRE_LEN_H			= r6

.def	DATEN_1_M			= r7
.def	DATEN_1_M_N		= r8

.def	DATEN_2_M			= r9
.def	DATEN_2_M_N		= r10

.def	PATTERN_TEMP 	= r11

.def  FLAGS         = r12

.def  RESERVE_1     = r13

.def  RESERVE_2     = r14
	
;--- higher Register ----------------------------------------------

.def	PRE_TRIGGER		= r16
.def	MODE					= r17

.def	TRIG_BYTE			= r18

.def	SREG_SAVE			= r19

.def	TIMER_L				= r20
.def	TIMER_H				= r21

.def	TRIG_MASK			= r22
.def	TIMER_WERT		= r23

.def	TEMP_1				= r24
.def	TEMP_0				= r25

; r26/r27, r28/r29, r30/r31 fr X,Y,Z reserviert 

;--- Macros -------------------------------------------------------

.macro  load_p                      ; ldt 16Bit @1 nach @0 High und @0 Low

        ldi     @0H,HIGH(@1)
        ldi     @0L,LOW(@1)

.endmacro

		
