
;**************************** Allgemeine Routinen **************************

;******************************* Sub-UART **********************************
; Sub: Sendet ein Byte aus TEMP_0 ber UART
; Parameter:    -
; Return:       -
; Scratch-Reg:  -
;***************************************************************************
uart_send_byte:
		push	TEMP_0

uart_send_byte_loop:
		lds		TEMP_0,UCSR0A
		sbrs  TEMP_0,UDRE0               ; Send-Register leer ?
    rjmp  uart_send_byte_loop        ; nein

		pop		TEMP_0
    sts   UDR0,TEMP_0
    ret

;******************************* Sub-UART **********************************
; Sub: Empfngt Byte ber UART
; Parameter:    -
; Return:       TEMP_0 Datenbyte
; Scratch-Reg:  -
;***************************************************************************

uart_receive_byte:
		lds		TEMP_0,UCSR0A			    	; warten auf Byte
		sbrs	TEMP_0,RXC0	
		rjmp 	uart_receive_byte				; nein

    lds		TEMP_0,UDR0							; holen


uart_receive_byte_end:
		ret

;*********************************************************************
; Sub: Send den Buffer-Inhalt
; Parameter:    Z zeigt auf erste Bufferposition
; Return:       -
; Scratch-Reg:  TEMP_0, Y, Z
;*********************************************************************

send_daten:
    tst     FLAGS
    brne    send_daten_end          ; Stop erkannt

		lds			YL,BUF_LEN_L						; alle Daten senden
		lds			YH,BUF_LEN_H
    cpi     YH,high(0x8000)         ; 32k?
    brne    send_daten_loop_1

    lsr     YH                      ; untere 16k senden
    ror     YL

send_daten_loop_1:
		in		TEMP_0,DATEN_1_IN					; Byte holen		
		and		TEMP_0,DATEN_1_M
		in		TEMP_1,DATEN_2_IN		
		and		TEMP_1,DATEN_2_M
		or		TEMP_0,TEMP_1	

		rcall	uart_send_byte	
		sbi		CLOCK_PORT,C_CLOCK				; Adresse weiterschalten
		cbi		CLOCK_PORT,C_CLOCK
		sbiw	Y,1	
		brne	send_daten_loop_1

		lds			YL,BUF_LEN_L						; alle Daten senden
		lds			YH,BUF_LEN_H
    cpi     YH,high(0x8000)         ; 32k?
    brne    send_daten_end          ; nein

    rcall   uart_receive_byte       ; auf Zeichen warten

    lsr     YH                      ; obere 16k senden
    ror     YL

send_daten_loop_2:
		in		TEMP_0,DATEN_1_IN					; Byte holen		
		and		TEMP_0,DATEN_1_M
		in		TEMP_1,DATEN_2_IN		
		and		TEMP_1,DATEN_2_M
		or		TEMP_0,TEMP_1	

		rcall	uart_send_byte	
		sbi		CLOCK_PORT,C_CLOCK				; Adresse weiterschalten
		cbi		CLOCK_PORT,C_CLOCK
		sbiw	Y,1	
		brne	send_daten_loop_2

send_daten_end:
		clr   FLAGS

    ret

;*********************************************************************
; Sub: Gibt den Receive-Interrupt frei fr Break
; Parameter:    -
; Return:       -
; Scratch-Reg:  TEMP_0
;*********************************************************************

enable_break:
		lds		TEMP_0,UCSR0A
		sbrs	TEMP_0,RXC0				    	; noch Daten zu holen?
		rjmp 	enable_break_ok					; nein

    lds		TEMP_0,UDR0						  ; holen
		rjmp	enable_break

enable_break_ok:
		sei														; Interrupt fr Braek freigeben	

		ret

;*********************************************************************
; Sub: Sendet ein Byte an die Shift-Register 
; Parameter:    -
; Return:       -
; Scratch-Reg:  TEMP_0, TEMP_1
;*********************************************************************

send_shift_daten:
		ldi		TEMP_1,8								; 8 Bit senden

send_shift_daten_loop:
		sbrs	TEMP_0,7
		cbi		SHIFT_PORT,SHIFT_DATA
		sbrc	TEMP_0,7
		sbi		SHIFT_PORT,SHIFT_DATA
		nop
		nop
		nop
		nop
		sbi		SHIFT_PORT,SHIFT_CLOCK
		nop
		nop
		nop
		nop
		cbi		SHIFT_PORT,SHIFT_CLOCK
		lsl		TEMP_0
		dec		TEMP_1
		brne	send_shift_daten_loop

		ret

;*********************************************************************
; Sub: Lscht den Buffer
; Parameter:    -
; Return:       -
; Scratch-Reg:  TEMP_0, Y
;*********************************************************************

clear_buffer:
		lds		YL,BUF_LEN_L
		lds		YH,BUF_LEN_H
		sbi		CTRL_PORT,C_SAMPLE			; Mode Sample

    rcall set_data_out

clear_buffer_loop:
		in		TEMP_0,DATEN_1_PORT			; Portstatus holen
		or		TEMP_0,DATEN_1_M				; Datenbits auf H setzeb
		out		DATEN_1_PORT,TEMP_0			; und raus
		in		TEMP_0,DATEN_2_PORT
		or		TEMP_0,DATEN_2_M
		out		DATEN_2_PORT,TEMP_0

		sbi		CLOCK_PORT,C_CLOCK
		cbi		CLOCK_PORT,C_CLOCK
		sbiw	Y,1
		brne	clear_buffer_loop

		cbi		CLOCK_PORT,C_CLOCK
		sts		TCCR1A,NULL

    rcall set_data_in

		cbi		CTRL_PORT,C_SAMPLE				; Mode Lesen

		ret

;*********************************************************************
; Sub: Setzt Timing und Pre-Trigger Position und gibt Break frei
; Parameter:    -
; Return:       -
; Scratch-Reg:  TEMP_0, Z, Y
;*********************************************************************

set_timing:
		rcall	clear_buffer

		load_p	Z,(2*tab_timer_intervall)

set_timing_loop:
		mov   TEMP_0,TIMER_WERT
    ldi   TEMP_1,6                ; Lnge eines Eintrags
    mul   TEMP_0,TEMP_1
    add   ZL,MUL_L
    adc   ZH,MUL_H           

		lpm		TIMER_L,Z+							; Timerwert holen
		lpm		TIMER_H,Z+

		lpm		YL,Z+										; Wert fr vollen Ram holen
		lpm		YH,Z+
		
		mov		XL,YL										; nach X kopieren fr Division / 8 (Pretrigger)
		mov		XH,YH

		lsr		XH
		ror		XL
		lsr		XH
		ror		XL
		lsr		XH
		ror		XL											; /8, 8 Abschnitte fr Pretrigger 

		mov		TEMP_0,PRE_TRIGGER

set_timer_trig_loop:
		cpi		TEMP_0,8								; = 100%
		breq	set_timer_trig_ok				; ja, ok

		inc		TEMP_0									; erhhen bis 8
		sub		YL,XL										; 1/8 abziehen
		sbc		YH,XH
		rjmp	set_timer_trig_loop

set_timer_trig_ok:						
		mov		PRE_LEN_L,YL
		mov		PRE_LEN_H,YH

		lpm		XL,Z+										; Wartezeit holen
		lpm		XH,Z

		; falls es externe 74LS164 gibt hier einfgen...

		mov		TEMP_0,RESERVE_2
		rcall	send_shift_daten				; zustzliches 2. LS164 rausschieben

		mov		TEMP_0,RESERVE_1
		rcall	send_shift_daten				; zustzliches 1. LS164 rausschieben

		mov		TEMP_0,TRIG_BYTE
		rcall	send_shift_daten				; Triggerbyte rausschieben

		mov		TEMP_0,TRIG_MASK
		rcall	send_shift_daten				; Triggermaske rausschieben

.if RAM_CLOCK == 50
	
		cpi		TIMER_WERT,2						; 20MHz oder grer?

.else

		cpi		TIMER_WERT,3						; 20MHz oder grer?

.endif

		brlo	set_timer_ok						; ja, externer Quarztakt

set_timer_ctc:
		in		TEMP_0,CTRL_PORT				; MUX auf intern setzen
		andi	TEMP_0,MUX_MASKE
		ori		TEMP_0,MUX_INT
		out		CTRL_PORT,TEMP_0

		ldi		TEMP_0,(1<<COM1A0)			; 16Bit-Timer auf Toggle OC1A
		sts		TCCR1A,TEMP_0

		sts		OCR1AH,TIMER_H					; Timerwert raus
		sts		OCR1AL,TIMER_L

		subi	TIMER_L,1	
		sbci	TIMER_H,0

		sts		TCNT1H,TIMER_H					; Counter-Register auf Timerwert -1 damit beim ersten Takt getoggelt wird
		sts		TCNT1L,TIMER_L

		ldi		TEMP_0,CLK_STOP					; CTC mit OCR1A, Prescale bleibt auf Stop
		sts		TCCR1B,TEMP_0

set_timer_ok:									

		rcall	enable_break						; RX-Interrupt zum Abbruch
					
		ret

;*********************************************************************
; Liest Daten, interner Takt, alle Triggervarianten
; Parameter:    -
; Return:       -
; Scratch-Reg:  TEMP_0, Z, Y, X
;*********************************************************************

int_clock_sample:

		rcall	set_timing
 	
		clt													    	; T als Trigger-Flag, erstmal lschen

		sbi		CTRL_PORT,C_SAMPLE		    	; Mode Sample
		cbi		CTRL_PORT,C_BUFFER		    	; Eingang aktiv
    cbi   CLOCK_PORT,TRIG_ENABLE      ; Trigger aktiv

.if RAM_CLOCK == 50

		cpi		TIMER_WERT,2						    ; 20/50MHz?
		brsh	int_clock_sample_int		    ; nein, max. 10MHz

		in		TEMP_0,CTRL_PORT
		andi	TEMP_0,MUX_MASKE
		ori		TEMP_0,MUX_20MHZ				    ; MUX auf 20MHz

		cpi		TIMER_WERT,1						    ; 20MHz?
		breq	int_clock_sample_20_50	    ; ja

		andi	TEMP_0,MUX_MASKE
		ori		TEMP_0,MUX_50MHz			    	; MUX auf 50MHz

.else

		cpi		TIMER_WERT,3						    ; 20/40/80MHz?
		brsh	int_clock_sample_int		    ; nein, max. 10MHz

		in		TEMP_0,CTRL_PORT
		andi	TEMP_0,MUX_MASKE
		ori		TEMP_0,MUX_20MHZ				    ; MUX auf 20MHz

		cpi		TIMER_WERT,2						    ; 20MHz?
		breq	int_clock_sample_20_50	    ; ja

		andi	TEMP_0,MUX_MASKE
		ori		TEMP_0,MUX_40MHz			    	; MUX auf 40MHz

		cpi		TIMER_WERT,1						    ; 40MHz?
		breq	int_clock_sample_20_50	    ; ja

		andi	TEMP_0,MUX_MASKE
		ori		TEMP_0,MUX_80MHz			    	; MUX auf 80MHz

.endif
    
int_clock_sample_20_50:
		out		CTRL_PORT,TEMP_0			    	; und Starten
		rjmp	int_clock_sample_loop

int_clock_sample_int:
		ldi		TEMP_1,CLK_START				    ; CTC mit OCR1A, Prescale 1
		sts		TCCR1B,TEMP_1

int_clock_sample_loop:                ; einfach nur auf Trigger warten...
		sbic	TRIGGER_PIN,TRIG_IN         ; #2
		set														    ; #1 Trigger erkannt
		brts	int_clock_sample_wait		    ; #1 T wird auch vom Break-IRQ gesetzt, Y ist dann 1!

    rjmp  int_clock_sample_loop       ; #2  ein Durchlauf 6 Takte -> 300ns, bei 50NHz max. 6 Takte Abweichung


int_clock_sample_wait:
		sbiw	Y,1										    	; #2 Rest-Zhler
		breq	int_clock_sample_end	    	; #1   --	fertig

		mov		ZL,XL									    	; #1 Wartezeit setzen
		mov		ZH,XH									    	; #1

int_clock_sample_wait_loop:
		nop                               ; #1
    sbiw	Z,1											    ; #2 noch warten?
		breq	int_clock_sample_wait 	    ; #2 nein - #1 ja

		nop                               ;           #1 - 10
		nop                               ;           #1
		nop                               ;           #1
		nop                               ;           #1
 
    rjmp  int_clock_sample_wait_loop  ;           #2

int_clock_sample_end:
		cli

		in		TEMP_0,CTRL_PORT            ; bei externer Clock wird hier gestoppt
		andi	TEMP_0,MUX_MASKE
		out		CTRL_PORT,TEMP_0

		ldi		TEMP_0,CLK_STOP					    ; CTC mit OCR1A, Prescale Stop
		sts		TCCR1B,TEMP_0               ; interner Clock stoppt hier

		cbi		CLOCK_PORT,C_CLOCK

		ldi		TEMP_0,0x00							    ; Toggle OC1A aus
		sts		TCCR1A,TEMP_0

    sbi   CLOCK_PORT,TRIG_ENABLE      ; Trigger inaktiv
		sbi		CTRL_PORT,C_BUFFER			    ; Eingang inaktiv
		cbi		CTRL_PORT,C_SAMPLE			    ; Mode Lesen

		ret

;*********************************************************************
; Sub: 0 Liest Daten, interner Takt, interner Trigger
; Parameter:    -
; Return:       -
; Scratch-Reg:  -
;*********************************************************************

int_clock_int_trigger:

		rcall	int_clock_sample
		
		ret 	

;*********************************************************************
; Sub: 2-8 Dummy
; Parameter:    -
; Return:       -
; Scratch-Reg:  -
;*********************************************************************
dummy:

    ret

;*********************************************************************
; Sub: Erzeugt Testpattern binary Counter
; Parameter:    -
; Return:       -
; Scratch-Reg:  Y, Z
;*********************************************************************

test_bin_count:
		ldi		TEMP_0,(1<<COM1A0)				; Toggle OC1A
		sts		TCCR1A,TEMP_0

    load_p  X,CLK_1M
		sts		OCR1AH,XH
		sts		OCR1AL,XL

    sbiw  X,1			                  ; TCNT auf 1 vor Compare setzen

		sts		TCNT1H,XH							    ; damit mit einem Takt ohne Verzgerung gestartet wid
		sts		TCNT1L,XL

		ldi		TEMP_0,CLK_STOP						; CTC mit OCR1A, Prescale Stop
		sts		TCCR1B,TEMP_0

		lds		YL,BUF_LEN_L
		lds		YH,BUF_LEN_H
		sbi		CTRL_PORT,C_SAMPLE				; Mode Sample

    rcall set_data_out

		ldi		TEMP_1,CLK_START					; CTC mit OCR1A, Prescale Start
		sts		TCCR1B,TEMP_1

test_bin_count_loop:
		in		TEMP_1,DATEN_1_PORT				; #1
		and		TEMP_1,DATEN_1_M_N				; #1  D0/D1 auf 0
		mov		TEMP_0,YL									; #1  Daten merken
		and		TEMP_0,DATEN_1_M					; #1  nur die beiden briglassen	
		or		TEMP_1,TEMP_0							; #1  mit den Portdaten zusammenfgen	
		out		DATEN_1_PORT,TEMP_1				; #1  und raus

		in		TEMP_1,DATEN_2_PORT				; #1
		and		TEMP_1,DATEN_2_M_N				; #1
		mov		TEMP_0,YL									; #1  Daten merken
		and 	TEMP_0,DATEN_2_M					; #1
		or		TEMP_1,TEMP_0							; #1
		out		DATEN_2_PORT,TEMP_1				; #1

		nop															; #1
		nop															; #1
		nop															; #1
		nop															; #1
		sbiw	Y,1												; #2
		brne	test_bin_count_loop				; #2	-> 20 Takte pro Loop = 1MHz

		ldi		TEMP_0,CLK_STOP						; CTC mit OCR1A, Prescale Stop
		sts		TCCR1B,TEMP_0

		cbi		CLOCK_PORT,C_CLOCK
		sts		TCCR1A,NULL

    rcall set_data_in

		cbi		CTRL_PORT,C_SAMPLE				; Mode Lesen

		ldi		PRE_TRIGGER,8

		ret

;*********************************************************************
; Sub: sendet Version
; Parameter:    -
; Return:       -
; Scratch-Reg:  Y, Z
;*********************************************************************

send_version:

		load_p	Z, (2*text_version)

send_version_loop:
		lpm		TEMP_0,Z+
		tst		TEMP_0
		breq 	send_version_ram_size

		rcall	uart_send_byte
		rjmp	send_version_loop

send_version_ram_size:
		ldi		TEMP_0,RAM_SIZE
		swap	TEMP_0							; High-Nibble
		andi	TEMP_0,0x0F
		ori		TEMP_0,'0'
    cpi   TEMP_0,'0'
    brne  send_version_ok

    ldi   TEMP_0,' '

send_version_ok:
		rcall	uart_send_byte

		ldi		TEMP_0,RAM_SIZE
		andi	TEMP_0,0x0F					; Low-Nibble
		ori		TEMP_0,'0'
		rcall	uart_send_byte

		ldi		TEMP_0,'k'
		rcall	uart_send_byte
		ldi		TEMP_0,'B'
		rcall	uart_send_byte


send_version_end:
		ret

;*********************************************************************
; Sub: setzt Datenpins auf Ausgang
; Parameter:    -
; Return:       -
; Scratch-Reg:  -
;*********************************************************************

set_data_out:
		in		TEMP_0,DATEN_1_DDR
		or		TEMP_0,DATEN_1_M
		out		DATEN_1_DDR,TEMP_0
		in		TEMP_0,DATEN_2_DDR
		or		TEMP_0,DATEN_2_M
		out		DATEN_2_DDR,TEMP_0

    ret

;*********************************************************************
; Sub: setzt Datenpins auf Eingang
; Parameter:    -
; Return:       -
; Scratch-Reg:  -
;*********************************************************************

set_data_in:
		in		TEMP_0,DATEN_1_DDR
		and		TEMP_0,DATEN_1_M_N
		out		DATEN_1_DDR,TEMP_0
		in		TEMP_0,DATEN_2_DDR
		and		TEMP_0,DATEN_2_M_N
		out		DATEN_2_DDR,TEMP_0

    ret

;*********************************************************************

;*********************************************************************
